# Generated by roxygen2: do not edit by hand

S3method(print,sobol_generator)
S3method(print,summary.sobol_generator)
S3method(summary,sobol_generator)
export(sobol_design)
export(sobol_dimensions)
export(sobol_generator)
export(sobol_index)
export(sobol_next)
export(sobol_next_n)
export(sobol_points)
export(sobol_skip_to)
importFrom(Rcpp,sourceCpp)
importFrom(methods,new)
useDynLib(sobol, .registration = TRUE)
