# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

sobol_points <- function(n, dim, skip = 0.0) {
    .Call(`_sobol_sobol_points`, n, dim, skip)
}

