% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{sobol_dimensions}
\alias{sobol_dimensions}
\title{Get Number of Dimensions of a Sobol Generator}
\usage{
sobol_dimensions(x, ...)
}
\arguments{
\item{x}{A sobol_generator object created by \code{\link{sobol_generator}}}

\item{...}{Additional arguments (currently unused)}
}
\value{
An integer representing the number of dimensions.
}
\description{
Returns the number of dimensions configured for the generator.
}
\examples{
gen <- sobol_generator(dimensions = 5)
dims <- sobol_dimensions(gen)
print(dims) # 5

}
