% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{summary.sobol_generator}
\alias{summary.sobol_generator}
\title{Summary Method for Sobol Generator}
\usage{
\method{summary}{sobol_generator}(object, ...)
}
\arguments{
\item{object}{A sobol_generator object}

\item{...}{Additional arguments passed to summary}
}
\value{
A list with class "summary.sobol_generator" containing summary
information
}
\description{
Summary Method for Sobol Generator
}
