% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol-package.R
\docType{package}
\name{sobol-package}
\alias{sobol}
\alias{sobol-package}
\title{sobol: Quasi-Monte Carlo Sobol Sequence Generator}
\description{
Provides a fast and efficient implementation of Sobol sequences for quasi-Monte Carlo methods. The Sobol sequence is a low-discrepancy sequence with the property that for all values of N, its subsequence x1, ..., xN has a low discrepancy. It can be used to generate quasi-random numbers for use in Monte Carlo integration and other simulation methods. This implementation is based on the algorithms described by Bratley and Fox (1988) \doi{10.1145/42288.214372} and uses direction numbers from Joe and Kuo (2008) \doi{10.1145/1358628.1358630}. The package includes both batch and incremental interfaces with support for arbitrary starting indices and reproducible sequences. It uses 'Rcpp' for efficient 'C++' integration.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://alrobles.github.io/sobol/}
  \item Report bugs at \url{https://github.com/alrobles/sobol/issues}
}

}
\author{
\strong{Maintainer}: Angel Robles \email{a.l.robles.fernandez@gmail.com}

Other contributors:
\itemize{
  \item Ilya M. Sobol (Original Sobol sequence algorithm) [contributor]
  \item Paul Bratley (Algorithm implementation reference) [contributor]
  \item Bennett L. Fox (Algorithm implementation reference) [contributor]
  \item Stephen Joe (Direction numbers and primitive polynomials) [contributor]
  \item Frances Y. Kuo (Direction numbers and primitive polynomials) [contributor]
}

}
\keyword{internal}
