% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{print.summary.sobol_generator}
\alias{print.summary.sobol_generator}
\title{Print Summary of Sobol Generator}
\usage{
\method{print}{summary.sobol_generator}(x, ...)
}
\arguments{
\item{x}{A summary.sobol_generator object}

\item{...}{Additional arguments passed to print}
}
\value{
Invisibly returns the input object
}
\description{
Print Summary of Sobol Generator
}
