% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sobol_generator.R
\name{print.sobol_generator}
\alias{print.sobol_generator}
\title{Print Method for Sobol Generator}
\usage{
\method{print}{sobol_generator}(x, ...)
}
\arguments{
\item{x}{A sobol_generator object}

\item{...}{Additional arguments passed to print}
}
\value{
Invisibly returns the input object (for chaining)
}
\description{
Print Method for Sobol Generator
}
